package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.config.PageUtils;
import digital.laboratory.platform.reagent.dto.AcceptanceRecordFormDTO;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.entity.AcceptanceRecordForm;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.mapper.AcceptanceRecordFormMapper;
import digital.laboratory.platform.reagent.service.AcceptanceRecordFormService;
import digital.laboratory.platform.reagent.service.BlacklistService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.vo.AcceptanceRecordFormVO;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * (验收记录表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (验收记录表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class AcceptanceRecordFormServiceImpl extends ServiceImpl<AcceptanceRecordFormMapper, AcceptanceRecordForm> implements AcceptanceRecordFormService {

    @Autowired
    private AcceptanceRecordFormService acceptanceRecordFormService;

    @Autowired
    private BlacklistService blacklistService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private OssFile ossFile;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Override
    public AcceptanceRecordForm commitForm(AcceptanceRecordFormDTO acceptanceRecordFormDTO, DLPUser dlpUser) {

        AcceptanceRecordForm byId = acceptanceRecordFormService.getById(acceptanceRecordFormDTO.getAcceptanceRecordFormId());

        if (byId.getStatus() != 0) {
            throw new RuntimeException(String.format("当前状态不能提交"));
        }
        BeanUtils.copyProperties(acceptanceRecordFormDTO, byId);

        byId.setUserName(dlpUser.getName());

        byId.setCommitTime(LocalDateTime.now());

        byId.setStatus(1);

        byId.setDateOfAcceptance(LocalDateTime.now());
        if (acceptanceRecordFormService.updateById(byId)) {
            return byId;
        } else {
            throw new RuntimeException(String.format("保存失败"));
        }
    }

    @Override
    public AcceptanceRecordForm addForm(String reagentConsumableId, String supplierId) {

        AcceptanceRecordForm acceptanceRecordForm = new AcceptanceRecordForm();

        acceptanceRecordForm.setId(IdWorker.get32UUID().toUpperCase());

        acceptanceRecordForm.setSupplierId(supplierId);

        acceptanceRecordForm.setReagentConsumableId(reagentConsumableId);

        ReagentConsumables byId = reagentConsumablesService.getById(reagentConsumableId);
        //将物品名称赋值给验收记录表
        acceptanceRecordForm.setReagentConsumableName(byId.getReagentConsumableName());

        acceptanceRecordForm.setStatus(0);

        if ((acceptanceRecordFormService.save(acceptanceRecordForm))) {

            return acceptanceRecordForm;

        } else throw new RuntimeException(String.format("创建验收任务失败"));
    }

    @Override
    public IPage<AcceptanceRecordFormVO> getAcceptanceRecordFormVOPage(IPage<AcceptanceRecordForm> page, QueryWrapper<AcceptanceRecordForm>qw) {

        IPage<AcceptanceRecordFormVO> acceptanceRecordFormVOPage = baseMapper.getAcceptanceRecordFormVOPage(page,qw);

        List<AcceptanceRecordFormVO> records = acceptanceRecordFormVOPage.getRecords();

        for (AcceptanceRecordFormVO record : records) {

            ReagentConsumables byId = reagentConsumablesService.getById(record.getReagentConsumableId());

            record.setReagentConsumables(byId);
        }
        return acceptanceRecordFormVOPage;

    }

    @Override
    public IPage<AcceptanceRecordFormVO> getAcceptanceRecordFormVORecordPage(IPage<AcceptanceRecordForm> page) {

        LambdaQueryWrapper<AcceptanceRecordForm> acceptanceRecordFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

        acceptanceRecordFormLambdaQueryWrapper.eq(AcceptanceRecordForm::getStatus, 4);

        List<AcceptanceRecordForm> list = acceptanceRecordFormService.list(acceptanceRecordFormLambdaQueryWrapper);

        ArrayList<AcceptanceRecordFormVO> acceptanceRecordFormVOS = new ArrayList<>();

        if (list.size() != 0) {

            for (AcceptanceRecordForm acceptanceRecordForm : list) {

                AcceptanceRecordFormVO acceptanceRecordFormVO = acceptanceRecordFormService.getAcceptanceRecordFormVO(acceptanceRecordForm.getId());

                acceptanceRecordFormVOS.add(acceptanceRecordFormVO);
            }
        }
        PageUtils pageUtils = new PageUtils();

        Page pages = pageUtils.getPages((int) page.getPages(), (int) page.getSize(), acceptanceRecordFormVOS);

        return pages;
    }

    @Override
    public AcceptanceRecordFormVO getAcceptanceRecordFormVO(String acceptanceRecordFormId) {

        AcceptanceRecordFormVO acceptanceRecordFormVO = baseMapper.getAcceptanceRecordFormVO(acceptanceRecordFormId);

        ReagentConsumables byId = reagentConsumablesService.getById(acceptanceRecordFormVO.getReagentConsumableId());

        acceptanceRecordFormVO.setReagentConsumables(byId);

        return acceptanceRecordFormVO;
    }

    @Override//一级审核
    public AcceptanceRecordForm primaryAudit(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        AcceptanceRecordForm byId = acceptanceRecordFormService.getById(auditAndApproveDTO.getUuId());

        if (byId.getStatus() != 1) {
            throw new RuntimeException(String.format("当前状态不能审核"));
        }
        byId.setPrimaryAuditorId(dlpUser.getId());
        byId.setAuditOpinionOfPrimary(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditTimeOfPrimary(LocalDateTime.now());
        byId.setAuditResultOfPrimary(auditAndApproveDTO.getAuditResult());

        if (auditAndApproveDTO.getAuditResult() == true) {
            byId.setStatus(2);
        } else byId.setStatus(-2);


        if (acceptanceRecordFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }

    @Override//二级审核
    public AcceptanceRecordForm secondaryAudit(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        AcceptanceRecordForm byId = acceptanceRecordFormService.getById(auditAndApproveDTO.getUuId());

        if (byId.getStatus() != 2) {
            throw new RuntimeException(String.format("当前状态不能审核"));
        }
        byId.setSecondaryAuditorId(dlpUser.getId());
        byId.setAuditOpinionOfSecondary(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditTimeOfSecondary(LocalDateTime.now());
        byId.setAuditResultOfSecondary(auditAndApproveDTO.getAuditResult());

        if (auditAndApproveDTO.getAuditResult() == true) {
            byId.setStatus(3);
        } else byId.setStatus(-3);

        if (acceptanceRecordFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }

    @Override//三级审核
    public AcceptanceRecordForm threeLevelAudit(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        AcceptanceRecordForm byId = acceptanceRecordFormService.getById(auditAndApproveDTO.getUuId());

        if (byId.getStatus() != 3) {
            throw new RuntimeException(String.format("当前状态不能审核"));
        }
        byId.setThreeLevelAuditorId(dlpUser.getId());
        byId.setAuditOpinionOfThreeLevel(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditTimeOfThreeLevel(LocalDateTime.now());
        byId.setAuditResultOfThreeLevel(auditAndApproveDTO.getAuditResult());

        if (auditAndApproveDTO.getAuditResult() == true) {

            if (byId.getNonconformingItem() != null) {

                blacklistService.addListById2(byId.getReagentConsumableId(), byId.getSupplierId());
            }
            byId.setStatus(4);
        } else byId.setStatus(-4);

        if (acceptanceRecordFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }

    @Override
    public void acceptanceRecordFormTablePDF(AcceptanceRecordFormVO acceptanceRecordFormVO, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("acceptanceRecordFormTablePDF.................");

        ReagentConsumables byId = reagentConsumablesService.getById(acceptanceRecordFormVO.getReagentConsumableId());

        if (acceptanceRecordFormVO.getTheSameBrandAndModel().equals("合格")) {
            acceptanceRecordFormVO.setTheSameBrandAndModelPrint(true);
        } else {
            acceptanceRecordFormVO.setTheSameBrandAndModelPrint(false);

        }
        if (acceptanceRecordFormVO.getConsistentQuantity().equals("合格")) {
            acceptanceRecordFormVO.setConsistentQuantityPrint(true);
        } else {
            acceptanceRecordFormVO.setConsistentQuantityPrint(false);

        }
        if (acceptanceRecordFormVO.getPackingInGoodCondition().equals("合格")) {
            acceptanceRecordFormVO.setPackingInGoodConditionPrint(true);
        } else {
            acceptanceRecordFormVO.setPackingInGoodConditionPrint(false);
        }
        if (acceptanceRecordFormVO.getValidityPeriod().equals("合格")) {
            acceptanceRecordFormVO.setValidityPeriodPrint(true);
        } else {
            acceptanceRecordFormVO.setValidityPeriodPrint(false);
        }
        if (acceptanceRecordFormVO.getDeliveryCycle().equals("合格")) {
            acceptanceRecordFormVO.setDeliveryCyclePrint(true);
        } else {
            acceptanceRecordFormVO.setDeliveryCyclePrint(false);
        }

        String id = acceptanceRecordFormVO.getId();

        String applyFileName = null;
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        if (byId.getCategory().equals("试剂") | byId.getCategory().equals("耗材")) {

            ossFile.fileGet("template" + "/" + "试剂耗材验收记录表模板.docx", bos);

            applyFileName = "试剂耗材验收记录表" + id;
        } else {

            ossFile.fileGet("template" + "/" + "标准物质验收记录表模板.docx", bos);

            applyFileName = "标准物质验收记录表" + id;
        }


        //-----------------------------
        // 生成 word 版本的 验收记录表

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("theSameBrandAndModelPrint", acceptanceRecordFormVO.getTheSameBrandAndModelPrint());
                    put("reagentConsumableName", acceptanceRecordFormVO.getReagentConsumableName());
                    put("dateOfAcceptance", LocalDateTimeUtil.format(acceptanceRecordFormVO.getDateOfAcceptance(),"yyyy年MM月dd日"));
                    put("supplierName",acceptanceRecordFormVO.getSupplierName());
                    put("consistentQuantityPrint",acceptanceRecordFormVO.getConsistentQuantityPrint());
                    put("packingInGoodConditionPrint",acceptanceRecordFormVO.getPackingInGoodConditionPrint());
                    put("validityPeriodPrint",acceptanceRecordFormVO.getValidityPeriodPrint());
                    put("deliveryCyclePrint",acceptanceRecordFormVO.getDeliveryCyclePrint());
                    put("nonconformingItem",acceptanceRecordFormVO.getNonconformingItem());
                    put("followUpTreatment",acceptanceRecordFormVO.getFollowUpTreatment());
                    put("acceptanceConclusion",acceptanceRecordFormVO.getAcceptanceConclusion());
                    put("secondaryAuditorName",acceptanceRecordFormVO.getSecondaryAuditorName());
                    put("auditOpinionOfSecondary",acceptanceRecordFormVO.getAuditOpinionOfSecondary());
                    put("auditTimeOfSecondary",LocalDateTimeUtil.format(acceptanceRecordFormVO.getAuditTimeOfSecondary(),"yyyy年MM月dd日"));
                    put("threeLevelAuditorName",acceptanceRecordFormVO.getThreeLevelAuditorName());
                    put("auditOpinionOfThreeLevel",acceptanceRecordFormVO.getAuditOpinionOfThreeLevel());
                    put("auditTimeOfThreeLevel",LocalDateTimeUtil.format(acceptanceRecordFormVO.getAuditTimeOfThreeLevel(),"yyyy年MM月dd日"));
                    put("id",acceptanceRecordFormVO.getId());
                    put("size",1);
                    put("bam_remarks",acceptanceRecordFormVO.getBam_remarks());
                    put("cq_remarks",acceptanceRecordFormVO.getCq_remarks());
                    put("pgc_remarks",acceptanceRecordFormVO.getPgc_remarks());
                    put("vp_remarks",acceptanceRecordFormVO.getVp_remarks());
                    put("dc_remarks",acceptanceRecordFormVO.getDc_remarks());
                    put("primaryAuditorName",acceptanceRecordFormVO.getPrimaryAuditorName());
                    put("auditOpinionOfPrimary",acceptanceRecordFormVO.getAuditOpinionOfPrimary());
                    put("auditTimeOfPrimary",LocalDateTimeUtil.format(acceptanceRecordFormVO.getAuditTimeOfPrimary(),"yyyy年MM月dd日"));
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "acceptanceRecordForm" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }


}
