package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.reagent.entity.CabinetForm;
import digital.laboratory.platform.reagent.entity.LatticeForm;
import digital.laboratory.platform.reagent.mapper.CabinetFormMapper;
import digital.laboratory.platform.reagent.service.CabinetFormService;
import digital.laboratory.platform.reagent.service.LatticeFormService;
import digital.laboratory.platform.reagent.vo.CabinetFormVO;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-22
 * @describe  服务实现类
 */
@Service
@RequiredArgsConstructor
public class CabinetFormServiceImpl extends ServiceImpl<CabinetFormMapper, CabinetForm> implements CabinetFormService {

    private CabinetFormService cabinetFormService;

    private LatticeFormService latticeFormService;

    @Transactional
    @Override//新增柜子
    public CabinetForm addById(CabinetForm cabinetForm){

        CabinetForm form = new CabinetForm();

        BeanUtils.copyProperties(cabinetForm,form);

        form.setId(IdWorker.get32UUID().toUpperCase());

        for (int i = 0; i < cabinetForm.getSpecification(); i++) {

            LatticeForm latticeForm = new LatticeForm();

            latticeForm.setCabinetFormId(form.getId());
            latticeForm.setId(IdWorker.get32UUID().toUpperCase());
            latticeForm.setNumber(form.getName()+"-"+(i+1));

            latticeFormService.save(latticeForm);
        }

        if (cabinetFormService.save(form)){
            return form;
        }else throw new RuntimeException(String.format("保存失败"));
    }

    @Override
    @Transactional//修改柜子信息
    public CabinetForm editById(CabinetForm cabinetForm){

        CabinetForm byId = cabinetFormService.getById(cabinetForm.getId());

        LambdaQueryWrapper<LatticeForm> latticeFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

        latticeFormLambdaQueryWrapper.eq(LatticeForm::getCabinetFormId,byId.getId());

        List<LatticeForm> list = latticeFormService.list(latticeFormLambdaQueryWrapper);
        //判断柜子是否为空
        for (LatticeForm latticeForm : list) {

            if (latticeForm.getStatus()==1){
                throw new RuntimeException(String.format("当前柜子有物品存放，不允许修改"));
            }
        }
        latticeFormService.removeBatchByIds(list);

        BeanUtils.copyProperties(cabinetForm,byId);
        //重新录入格子信息
        for (int i = 0; i < cabinetForm.getSpecification(); i++) {

            LatticeForm latticeForm = new LatticeForm();

            latticeForm.setCabinetFormId(byId.getId());
            latticeForm.setId(IdWorker.get32UUID().toUpperCase());
            latticeForm.setNumber(byId.getName()+"-"+(i+1));

            latticeFormService.save(latticeForm);
        }

        if (cabinetFormService.updateById(cabinetForm)){
            return byId;
        }else throw new RuntimeException(String.format("保存失败"));
    }

    @Override//通过ID查询柜子信息
    public CabinetFormVO getFormById(String id){

        CabinetForm byId = cabinetFormService.getById(id);

        LambdaQueryWrapper<LatticeForm> latticeFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

        latticeFormLambdaQueryWrapper.eq(LatticeForm::getCabinetFormId,id);

        List<LatticeForm> list = latticeFormService.list(latticeFormLambdaQueryWrapper);

        CabinetFormVO cabinetFormVO = new CabinetFormVO();

        BeanUtils.copyProperties(byId,cabinetFormVO);

        cabinetFormVO.setLatticeFormList(list);

        return cabinetFormVO;
    }

    @Override
    @Transactional
    public Boolean delById(String id){

        CabinetForm byId = cabinetFormService.getById(id);

        LambdaQueryWrapper<LatticeForm> latticeFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

        latticeFormLambdaQueryWrapper.eq(LatticeForm::getCabinetFormId,id);

        List<LatticeForm> list = latticeFormService.list(latticeFormLambdaQueryWrapper);

        for (LatticeForm latticeForm : list) {

            if (latticeForm.getStatus()==1){
                throw new RuntimeException(String.format("当前柜子有物品存放，不允许修改"));
            }
        }
        latticeFormService.removeBatchByIds(list);

        if (cabinetFormService.removeById(byId)){
            return true;
        }else throw new RuntimeException(String.format("删除失败"));
    }

}
